/*
 *  TOPPERS/JSP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Just Standard Profile Kernel
 * 
 *  Copyright (C) 2000 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 * 
 *  嵭Ԥϡʲξ˸¤ꡤܥեȥܥ
 *  եȥѤΤޤࡥʲƱˤѡʣѡ
 *  ۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥХʥꥳɤηޤϵȤ߹
 *      ѤˤϡΤ줫ξȡ
 *    (a) ѤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) Ѥη֤̤ˡˤäơ嵭Ԥ𤹤
 *        ȡ
 *  (3) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭Ԥդ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥϡ
 *  ܥեȥ˴ؤơŬѲǽޤơʤݾڤԤ
 *  ʤޤܥեȥѤˤľŪޤϴŪ
 *  ʤ»˴ؤƤ⡤Ǥʤ
 * 
 *  @(#) $Id: id.cpp,v 1.3 2000/11/14 16:57:33 takayuki Exp $
 */

//------------------------------------------------------------
// ID - class implementation
//
//   class ID is a type for managing objects' identifier and
//		assigning suitable identifier automatically for new
//		object.

#ifdef _MSC_VER
#pragma warning(disable:4786)
#endif

#include "id.h"
#include <algorithm>

using namespace std;

ID::ID(const long Size)
{
	LowerBound = 1;
	UpperBound = Size -1 + LowerBound;
	AssignedIdentifier.clear();
}

ID::ID(long Lower, unsigned long Size)
{
	LowerBound = Lower;
	UpperBound = Size -1 + LowerBound;
	AssignedIdentifier.clear();
}

ID::ID(long Lower, long Upper)
{
	if(Lower > Upper)
	{
		Lower ^= Upper;
		Upper ^= Lower;
		Lower ^= Upper;
	}
	LowerBound = Lower;
	UpperBound = Upper;
	AssignedIdentifier.clear();
}

bool ID::Assign(long value)
{
	if(value < LowerBound || value > UpperBound)
		return false;

	return (AssignedIdentifier.insert(value)).second;
}

	//METHOD Resign
	// value : The value of ID you want remove
	// Result: Execution status (True means success)
bool ID::Resign(long value)
{
	if(value < LowerBound || value > UpperBound)
		return false;

	AssignedIdentifier.erase(value);
	return true;
}

	//METHOD GetMinID
	// Result: Maximum value of ID existed
long ID::GetMaxID(void)
{
	set<long>::iterator maxvalue;

	maxvalue = max_element(AssignedIdentifier.begin(),AssignedIdentifier.end());
	if(maxvalue == AssignedIdentifier.end())
		return LowerBound -1;

	return *maxvalue;
}

	//METHOD GetMinID
	// Result: Minimum value of ID existed 
long ID::GetMinID(void)
{
	set<long>::iterator minvalue;

	minvalue = min_element(AssignedIdentifier.begin(),AssignedIdentifier.end());
	if(minvalue == AssignedIdentifier.end())
		return UpperBound + 1;

	return *minvalue;
}

	//METHOD GetNewID
	// Result: Suitable New ID
long ID::GetNewID(void)
{
	long prev;
	set<long>::iterator scope;

	if(AssignedIdentifier.size()==0)
		return LowerBound;

	scope = AssignedIdentifier.end();
	scope --;
	if(*scope-LowerBound+1 == AssignedIdentifier.size())
	{
		if(*scope+1>UpperBound)
			return LowerBound -1;
		else
			return *scope+1;
	}

	scope = AssignedIdentifier.begin();
	if(*scope != LowerBound)
		return LowerBound;
	
	prev = *scope;
	scope++;
	while(scope != AssignedIdentifier.end() )
	{
		if(prev+1<*scope)
			return prev+1;
		prev = *scope;
		scope++;
	}

	return LowerBound-1;
}

	//METHOD IsAvailable
	//  Result: Status whether this array is available or not
bool ID::IsAvailable(void)
{
	long prev;
	set<long>::iterator work;

		//Empty check (Empty ID set is not bad, so result should be true.)
	if(AssignedIdentifier.size() == 0)
		return true;

		//Smallest ID should be equal to lower bound
	work = AssignedIdentifier.begin();
	if(*work != LowerBound)
		return false;

		//The array of IDs must not include any cracks
	prev = *work;
	work ++;
	while(work != AssignedIdentifier.end() )
	{
		if(*work != prev +1)
			return false;
		
		prev = *work;
		work++;
	}

	return true;
}


//---
// Degined and Programmed by S.D.F. in Oct,2000